package cds.aida.gui;
/* $Id: JToolsTree.java,v 1.1 2007/06/04 14:18:21 beugnot Exp $ */
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceListener;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import cds.aida.tool.Tool;

/**
 * Part of the ToolsView handling the tree representation
 * and drag'n'drop facilities
 */
public class JToolsTree extends JTree
implements DragGestureListener, DragSourceListener {
	public JToolsTree() {
		super();
		DragSource dragSource=DragSource.getDefaultDragSource();
		dragSource.createDefaultDragGestureRecognizer(
				this/* is the component target*/,
				DnDConstants.ACTION_COPY_OR_MOVE,
				this/* drag gesture is listening*/
				);
	}
	/* (non-Javadoc)
	 * @see java.awt.dnd.DragGestureListener#dragGestureRecognized(java.awt.dnd.DragGestureEvent)
	 */
	public void dragGestureRecognized(DragGestureEvent dge) {
		TreePath path=getLeadSelectionPath();
		if(path==null) { return; } // not sure this can actually happen.
		DefaultMutableTreeNode node=
			(DefaultMutableTreeNode)path.getLastPathComponent();
		Object obj=node.getUserObject();
		if(obj instanceof Tool) {
			dge.startDrag(
					DragSource.DefaultCopyDrop,
					(Transferable)obj/* is transferable*/,
					this/* drag source is listening*/);
		}
		else if(obj instanceof String) {
			dge.startDrag(
					DragSource.DefaultCopyDrop,
					new StringSelection((String)obj),
					this/* drag source is listening*/);
		}
		return;
	}
	/* (non-Javadoc)
	 * @see java.awt.dnd.DragSourceListener#dragDropEnd(java.awt.dnd.DragSourceDropEvent)
	 */
	public void dragDropEnd(DragSourceDropEvent dsde) {
	}
	/* (non-Javadoc)
	 * @see java.awt.dnd.DragSourceListener#dragEnter(java.awt.dnd.DragSourceDragEvent)
	 */
	public void dragEnter(DragSourceDragEvent dsde) {
	}
	/* (non-Javadoc)
	 * @see java.awt.dnd.DragSourceListener#dragExit(java.awt.dnd.DragSourceEvent)
	 */
	public void dragExit(DragSourceEvent dse) {
	}
	/* (non-Javadoc)
	 * @see java.awt.dnd.DragSourceListener#dragOver(java.awt.dnd.DragSourceDragEvent)
	 */
	public void dragOver(DragSourceDragEvent dsde) {
	}
	/* (non-Javadoc)
	 * @see java.awt.dnd.DragSourceListener#dropActionChanged(java.awt.dnd.DragSourceDragEvent)
	 */
	public void dropActionChanged(DragSourceDragEvent dsde) {
	}
	
    /**
     * Gets the selected node.
     * @return the userObject of this node
     */
    public Object getSelection() {
	DefaultMutableTreeNode selected = 
	    (DefaultMutableTreeNode) getLastSelectedPathComponent();

	if(selected!=null)
	    return selected.getUserObject();
	else
	    return null;
    }
}
