package cds.aida.gui;
/* $Id: WS1View.java,v 1.1 2007/06/04 14:18:21 beugnot Exp $ */
import java.awt.BorderLayout;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.BorderFactory;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.border.Border;


import cds.aida.tool.Tool;
import cds.aida.tool.ToolPort;
import cds.aida.tool.ToolsList;
import cds.aida.w1.ToolCallStatus;
import cds.aida.w1.ToolView;
import cds.aida.w1.ToolViewFactory;
import cds.aida.w1.WS1;
import cds.jlow.client.descriptor.Attributs;
import cds.jlow.client.descriptor.Registerer;
import cds.jlow.client.graph.Jlow;
import cds.jlow.descriptor.IDescriptor;
import cds.jlow.descriptor.ITaskDescriptor;
import cds.jlow.descriptor.Register;
import cds.jlow.util.Utils;

/**
 *  View for a WS1, which if a workspace containing only one tool.
 *  <p>GUI for the input/output/execution of the tool selected in the WS1
 */
public class WS1View extends JPanel
implements WSView, Observer, DropTargetListener {
	/** workspace model for this view */
	protected WS1 ws;
	private Tool t;
	private Register regW1, reg;
	private Action
	actionLaunch, actionResetFields, actionShowCgiRequest;
	// visu
	JButton launch,resetf,showcgiurl;
	private Border compborder=BorderFactory.createEmptyBorder(10,10,10,10);
		
	public WS1View(WS1 ws,ToolsList tl) {
		// set the workspace which is viewed and observed
		this.ws=ws;
		regW1 = new Registerer();
		reg = tl.getReg();
		ws.addObserver(this);
		ws.addGraphDescriptorListener(new CustomGDListener());
		this.initActions();
		
		// content until a tool is added
		JLabel pname=new JLabel();
		pname.setText(L10N.getInstance().get("ws1.name", "Workspace WS1")); //$NON-NLS-1$ //$NON-NLS-2$
		pname.setBorder(compborder);
		setLayout(new BorderLayout());
		add(pname, BorderLayout.NORTH);
		
		// drag'n'drop facilities
		new DropTarget(
				this/* is the target component*/,
		        DnDConstants.ACTION_COPY_OR_MOVE,
		        this/* drop target is listening*/
				);
	}

	private void initActions() {
		ActionMap am=getActionMap();
		InputMap im=getInputMap();
		actionLaunch=new ActionLaunch();
		am.put("launch",actionLaunch); //$NON-NLS-1$
		actionResetFields=new ActionResetFields();
		am.put("resetfields",actionResetFields); //$NON-NLS-1$
		actionShowCgiRequest=new ActionShowCgiRequest();
		am.put("showcgirequest",actionShowCgiRequest); //$NON-NLS-1$
	}
	
	/**
	 * @return cgi request string (server+params)
	 */
	public String getCgiRequest() {
		Logger logger=Logger.getInstance();
		Conf conf=Conf.getInstance();
		String id=t.getIdOT();
		URL callerurl=conf.getAidaCgi(Conf.CGI_USETOOL); // XXX monosrv
		String caller=callerurl.toString();
		caller+="?tool.id="+id; //$NON-NLS-1$
				
		Iterator iter=t.getInputPort().iterator();
		ToolPort ti;
		int tin=1;
		while(iter.hasNext()) {
			ti = (ToolPort)regW1.getDescriptor(iter.next());
			
			if(ti.isReady()) {
				// Construct call string of input.X
				// possibly multi-valued
				if(ti.isMultivalued()) {
					Enumeration tivals=ti.getValues();
					int tivi=1;
					while(tivals.hasMoreElements()) {
						String tival=tivals.nextElement().toString();
						String tivalenc=t.encodeInputValue(tival);
						caller+="&input."+tin+"."+tivi+"="+tivalenc; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						tivi++;
					}
				}
				else {
					String tival=ti.getFirstValue().toString();
					String tivalenc=t.encodeInputValue(tival);
					caller+="&input."+tin+"="+tivalenc; //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
			else {
				logger.log(Logger.TRACE,"Tool.call: input("+tin+") not set"); //$NON-NLS-1$ //$NON-NLS-2$
				t.setCallStatus(ToolCallStatus.CALL_INPUT_NOT_READY);
				return ""; //$NON-NLS-1$
			}
			tin++;
		}
		return caller;
	}
	
	/**
	 * Parse a digest (text returned after a call),
	 * via parseDigestLine
	 * @return parsing success
	 */
	protected boolean parseDigest() {
		Logger logger=Logger.getInstance();
		// iokdbg: resetOutputs here because of possiblity of multiple launch
		// should probably be handled before this point, thought. 
		resetOutputs();
		if(t.getDigest()==null||t.getDigest().equals("")) { //$NON-NLS-1$
			logger.log(
					Logger.ERROR,
					"Tool.parseDigest:empty"); //$NON-NLS-1$
			return false;
		}
		logger.log(Logger.DEBUG, "Tool.parseDigest(\n"+t.getDigest()+")"); //$NON-NLS-1$ //$NON-NLS-2$
		try {
			boolean parsed_ok=true;
			BufferedReader br=new BufferedReader(new StringReader(t.getDigest()));
			String l=br.readLine();

			while(l!=null) {
				l=l.trim();
				if(l.equals("")) { //$NON-NLS-1$
					l=br.readLine();
					continue;
				}
				// --- mark the beginning of debug/verbose infos, if any.
				if(l.startsWith("---")) { //$NON-NLS-1$
					break;
				}
				parsed_ok=parsed_ok && parseDigestLine(l);
				l=br.readLine();
			}
			return parsed_ok;
		}
		catch(IOException ioe) {
			logger.log(
					Logger.ERROR,
					"Tool.parseDigest:IOException"); //$NON-NLS-1$
			t.setCallStatus(ToolCallStatus.CALL_PARSERESULTS_ERROR);
			// XXX handle more precisely ?
			return false;
		}
	}
	
	/**
	 * parse a digest line, sets the tool output accordingly
	 * @param l - digest line
	 * @return true on parsing success
	 */
	protected boolean parseDigestLine(String l) {
		// multiple return point on error
		boolean parsed_ok=true;
		Logger logger=Logger.getInstance();
		
		if(l.startsWith("Error")) { // XXX hardcoded //$NON-NLS-1$
			logger.log(
					Logger.ERROR,
					"Tool.parseDigestLine: "+l); //$NON-NLS-1$
			// XXX error transmission
			return true;
		}
		// split key=value
		String kv[]=l.trim().split("=",2); //$NON-NLS-1$
		if(kv.length!=2) {
			logger.log(
					Logger.WARNING,
					"Tool.parseDigestLine: line("+l+") badly formed"); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		String value=kv[1];
		
		// split key.subkey
		String key[]=kv[0].split("[.]"); //$NON-NLS-1$
		if(key.length<2) {
			logger.log(
					Logger.WARNING,
					"Tool.parseDigestLine: key of line("+l+") badly formed"); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}

		if(key[0].equals("output")) { //$NON-NLS-1$
			if(value.startsWith("FILE_SEQUENCE")) { //$NON-NLS-1$
				// skip this one, values digest lines will be parsed latter
				return parsed_ok;
			}
			
			if(key[1].equals("status")) { //$NON-NLS-1$
				// XXX output.status handling
				logger.log(
						Logger.INFO,
						"Tool.parseDigestLine: Status("+value+")"); //$NON-NLS-1$ //$NON-NLS-2$
				return true;
			}
			if(key[1].equals("stdout")) { //$NON-NLS-1$
				t.stdout().addValue(value);
			}
			else if(key[1].equals("stderr")) { //$NON-NLS-1$
				t.stderr().addValue(value);
			}
			else {
				// TODO: chk if subkeys
				int idx;
				try {
					idx=new Integer(key[1]).intValue();
//					ERREUR (POSSIBILITÉES : AJOUTER LE REGISTRE AU TOOL OU DÉPLACER L'EXÉCUTION DU TOOL DANS UN ENDROIT OU LE REGISTRE EXISTE)
					ToolPort tp = (ToolPort)regW1.getDescriptor(t.getOutput(idx));
					tp.addValue(value);
				}
				catch(Exception e) {
					// NumberFormatException if index malformed
					// NullPointerException if output(idx) out of bounds
					logger.log(
							Logger.ERROR,
							"Tool.parseDigestLine: index("+key[1]+") error"); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		}
		else {
			logger.log(
					Logger.WARNING,
					"Tool.parseDigestLine: line("+l+") unknown, skip."); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return parsed_ok;
	}

	/**
	 * reset every IO, and stdout/err
	 */
	public void reset() {
		t.setDigest(""); //$NON-NLS-1$
		t.stdout().clearValues();
		t.stderr().clearValues();
		t.setCallStatus(ToolCallStatus.CALL_NOTCALLED);
		resetInputs();
		resetOutputs();
		t.getObs().ping();
	}
	

	
	public void resetInputs() {
		Vector v;
		Enumeration e;
		v=t.getListIPort();
		e=v.elements();
		while(e.hasMoreElements()) {
			ToolPort io=(ToolPort)regW1.getDescriptor(e.nextElement());
			io.clearValues();
		}	
		// obs.ping() will be done at the ToolIO level
	}
	 

	
	public void resetOutputs() {
		Vector v;
		Enumeration e;
		v=t.getListOPort();
		e=v.elements();
		while(e.hasMoreElements()) {
			ToolPort io=(ToolPort)regW1.getDescriptor(e.nextElement());
			io.clearValues();
		}
		t.stdout().clearValues();
		t.stderr().clearValues();
		// obs.ping() will be done at the ToolIO level
	}
	
	/**
	 * @return cgi request string (server+params)
	 */
	
	public void run() {
		Logger logger=Logger.getInstance();
		Conf conf=Conf.getInstance();
		resetOutputs();
		t.setCallStatus(ToolCallStatus.CALL_CALLING);
		String id=t.getIdOT();
		logger.log(Logger.TRACE,"Tool.call("+id+")"); //$NON-NLS-1$ //$NON-NLS-2$
		String caller=getCgiRequest();
		if("".equals(caller)) { //$NON-NLS-1$
			// Proper call status has been set in getCgiRequest
			return;
		}
		
		// XXX should set output_format as resultlocation, get location,
		// and wait til unlocked to get digest, so that no http behaviour pb.
		
		URL url=null;
		URLConnection urlc=null;
		String mime;
		try {
			url=new URL(caller);
			t.setCallStatus(ToolCallStatus.CALL_CONNECTING);
			// XXX iokdbg: try to fix timeout, with sthg like:
			// System.setProperty ("sun.net.client.defaultReadTimeout","30000");
			// @see http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4143518
			urlc=url.openConnection();
			urlc.setUseCaches(false);
			urlc.setRequestProperty("User-Agent",conf.getUserAgent()); //$NON-NLS-1$
		}
		catch(MalformedURLException mue) {
			logger.log(Logger.ERROR, "Tool.call: Malformed Caller Url"); //$NON-NLS-1$
			logger.log(Logger.DEBUG, "Tool.call: caller="+caller); //$NON-NLS-1$
			t.setCallStatus(ToolCallStatus.CALL_BAD_URL);
			return;
		}
		catch(IOException ioe) {
			logger.log(Logger.ERROR, "Tool.call: Error connecting"); //$NON-NLS-1$
			logger.log(Logger.DEBUG, "Tool.call: caller="+caller); //$NON-NLS-1$
			t.setCallStatus(ToolCallStatus.CALL_CONNECTION_ERROR);
			return;
		}
		
		boolean called_ok=false;
		try {
			t.setCallStatus(ToolCallStatus.CALL_GETCONTENT);
			t.setDigest((String)urlc.getContent());
		}
		catch(IOException ioe) {
			logger.log(Logger.ERROR,"Tool.call: Error getting result()"); //$NON-NLS-1$
			String ioerr=ioe.getMessage();
			logger.log(Logger.DEBUG,"Tool.call: "+ioerr); //$NON-NLS-1$
			if(ioerr.indexOf("HTTP response code: 500")!=-1) { //$NON-NLS-1$
				t.setCallStatus(ToolCallStatus.CALL_GETCONTENT_SERVER_ERROR);
			}
			else {
				t.setCallStatus(ToolCallStatus.CALL_GETCONTENT_ERROR);
			}
			return;
		}
		catch(Exception e) {
			logger.log(Logger.DEBUG, "Tool.call: UnsupportedMime"); //$NON-NLS-1$
			// XXX: handle other mime types if any
			t.setCallStatus(ToolCallStatus.CALL_MIMETYPE_ERROR);
			return;
		}

		t.setCallStatus(ToolCallStatus.CALL_PARSERESULTS);
		called_ok=this.parseDigest();
		if(!called_ok) {
			logger.log(Logger.DEBUG, "Tool.call: !parseDigest"); //$NON-NLS-1$
			t.setCallStatus(ToolCallStatus.CALL_PARSERESULTS_ERROR);
			return;
		}
		t.setCallStatus(ToolCallStatus.CALL_OK);
	}
	
	class ActionLaunch extends AbstractAction {
		public ActionLaunch() {
			L10N lion=L10N.getInstance();
			putValue(NAME,lion.get("tool.run.label","Launch")); //$NON-NLS-1$ //$NON-NLS-2$
			//putValue(MNEMONIC_KEY,new Integer(KeyEvent.VK_L));
			putValue(ACCELERATOR_KEY,KeyStroke.getKeyStroke("F5")); //$NON-NLS-1$
		}
		public void actionPerformed(ActionEvent e) {
			// XXX should be in WS1 rather than WS1View ?
			// or even elsewhere ?
			Tool t=(Tool)ws.getTool();
			launch.setEnabled(false);
			resetf.setEnabled(false);
			run();
//			new Thread(t).start();
		}
	}
	class ActionShowCgiRequest extends AbstractAction {
		public ActionShowCgiRequest() {
			L10N lion=L10N.getInstance();
			putValue(NAME,lion.get("ws1.showcgirequest","Show Request")); //$NON-NLS-1$ //$NON-NLS-2$
		}
		public void actionPerformed(ActionEvent e) {
			Tool t=(Tool)ws.getTool();
			String req=getCgiRequest();
			JFrame f=new JFrame();
			JPanel p=new JPanel();
			p.setLayout(new BorderLayout());
			JTextField tf=new JTextField(req);
			p.add(tf,BorderLayout.CENTER);
			f.getContentPane().add(p);
			f.pack();
			f.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			f.setVisible(true);
		}
	}
	class ActionResetFields extends AbstractAction {
		public ActionResetFields() {
			L10N lion=L10N.getInstance();
			putValue(NAME,lion.get("ws1.resetfields.label","Clear")); //$NON-NLS-1$ //$NON-NLS-2$
		}
		public void actionPerformed(ActionEvent e) {
			reset();
		}
	}
	

	/**
	 * GUI for actions (launch tool, etc.)
	 * @return Toolbar
	 */
	protected JToolBar actionsToolBar() {
		launch=new JButton(actionLaunch);
		resetf=new JButton(actionResetFields);
		showcgiurl=new JButton(actionShowCgiRequest);
		JToolBar actions=new JToolBar(JToolBar.HORIZONTAL);
		actions.setFloatable(false);
		actions.setOpaque(false);
		actions.add(launch);
		actions.add(resetf);
		actions.add(showcgiurl);
		return actions;
	}
	
	/**
	 * wrap the toolInterface in a scrollable pane
	 * @return GUI
	 */
	protected JScrollPane scrollableToolInterface() {
		//Tool's cloning
		Tool rifeTl = ws.getTool();
		t = (Tool)rifeTl.clone();
		String id = Utils.getIdUnique();
		regW1.putDescriptor(id,t);

		
		// Add input port to the new task
		java.util.List ports = ((Tool) rifeTl).getInputPort();

		for (Iterator it = ports.iterator(); it.hasNext();) {
			Object port = it.next();

			if (port != null) {

				ToolPort portdesc = (ToolPort)reg.getDescriptor(port);
				// Creates all ports
				regW1.putDescriptor(id=Utils.getIdUnique(),(ToolPort) portdesc.clone());
				
				t.addInputPort(id);
			}
		}

		// Add output port to the new task
		ports = ((Tool) rifeTl).getOutputPort();

		for (Iterator it = ports.iterator(); it.hasNext();) {
			Object port = it.next();

			if (port != null) {

				ToolPort portdesc = (ToolPort)reg.getDescriptor(port);
				// Creates all ports
				regW1.putDescriptor(id=Utils.getIdUnique(),(ToolPort) portdesc.clone());
				
				t.addOutputPort(id);
			}
		}
		
		
		ToolView tv=ToolViewFactory.create(t,regW1);
		JScrollPane sp=new JScrollPane(tv);
		// sets mouse wheel scroll speed
		sp.getVerticalScrollBar().setUnitIncrement(10);
		// starts on top of pane
		sp.getVerticalScrollBar().setValue(0);
		return sp;
	}
	
	/**
	 * update when notified by the observable workspace
	 */
	public void update(Observable o, Object oo) {
		Logger.getInstance().log(
				Logger.TRACE,
				"WS1View.update("+oo.getClass()+")"); //$NON-NLS-1$ //$NON-NLS-2$
		
		if(oo instanceof WS1.ObservableWorkSpace) {
			// iokdbg: remove/recreate shouldnt be the updating process
			this.removeAll();
			JScrollPane sp=scrollableToolInterface();
			JToolBar tb=actionsToolBar();
			this.add(tb,BorderLayout.NORTH);
			this.add(sp,BorderLayout.CENTER);
			Conf.getInstance().getGraphicRoot().updateUI();
			// XXX right place to do this ? how to unregister ?
			Tool t=ws.getTool();
			if(t!=null) { t.addObserver(this); }
		}
		
		else if(oo instanceof Tool) {
			Tool t=(Tool)oo;
			Tool tt=ws.getTool();
			if(tt==oo) { // iokdbg: couldnt unreg. from obs. old tool
				boolean b=t.getCallStatus().isFinished();
				launch.setEnabled(b);
				resetf.setEnabled(b);
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see java.awt.dnd.DropTargetListener#dragEnter(java.awt.dnd.DropTargetDragEvent)
	 */
	public void dragEnter(DropTargetDragEvent dtde) {
	}
	/* (non-Javadoc)
	 * @see java.awt.dnd.DropTargetListener#dragExit(java.awt.dnd.DropTargetEvent)
	 */
	public void dragExit(DropTargetEvent dte) {
	}
	/* (non-Javadoc)
	 * @see java.awt.dnd.DropTargetListener#dragOver(java.awt.dnd.DropTargetDragEvent)
	 */
	public void dragOver(DropTargetDragEvent dtde) {
	}
	/* (non-Javadoc)
	 * @see java.awt.dnd.DropTargetListener#drop(java.awt.dnd.DropTargetDropEvent)
	 */
	public void drop(DropTargetDropEvent dtde) {
		// drag'n'drop facilities
		try {
			// only a Tool is dropable here
			DataFlavor df=new Tool.ToolDataFlavor();
			Transferable tr=dtde.getTransferable();
			if(dtde.isDataFlavorSupported(df)) {
				Tool tool=(Tool)tr.getTransferData(df);
				dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
				// effective drop in the workspace
				ws.addTool(tool);
				//update();
				dtde.dropComplete(true);
			}
			else {
				dtde.rejectDrop();
			}
		}
		catch(IOException ioe) {
			ioe.printStackTrace();
		}
		catch(UnsupportedFlavorException ufe) {
			ufe.printStackTrace();
		}
	}
	/* (non-Javadoc)
	 * @see java.awt.dnd.DropTargetListener#dropActionChanged(java.awt.dnd.DropTargetDragEvent)
	 */
	public void dropActionChanged(DropTargetDragEvent dtde) {
	}
	/**
	 * @return Returns the ws.
	 */
	public WS1 getWs() {
		return ws;
	}
	/**
	 * @param ws The ws to set.
	 */
	public void setWs(WS1 ws) {
		this.ws = ws;
		//update(null,null);
	}
}
