package cds.aida.tool;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import cds.aida.gui.Logger;
import cds.aida.w1.ToolPortViewFiles;
import cds.aida.w1.ToolPortViewString;
import cds.jlow.descriptor.BooleanDescriptor;
import cds.jlow.descriptor.Constant;
import cds.jlow.descriptor.IPortDescriptor;
import cds.jlow.descriptor.IntegerDescriptor;
import cds.jlow.descriptor.PortDescriptor;
import cds.jlow.util.Type;
import cds.jlow.util.Utils;


public class ToolPort extends PortDescriptor {
	private String comment;
	private String defaultValue;
//	private Vector f = new Vector();
	
	
	protected ObservableToolPort obs;
	
	//One Tool
	int index=-1;
//	private String s="";
	
	public ToolPort(){
		super();
		obs=new ObservableToolPort(this);
	}
	
	public ToolPort(String id, String name){
		super(id,name);
		obs=new ObservableToolPort(this);
	}
	
	public ToolPort(String id){
		super(id,"");
		obs=new ObservableToolPort(this);
	}
	
	public ToolPort(String id, String name, String comment){
		super(id,name);
		setComment(comment);
		obs=new ObservableToolPort(this);
	}
	
	public ToolPort(String id, String n, String p, String comment){
		super(id,n,p);
		setComment(comment);
		obs=new ObservableToolPort(this);
	}
	
	protected boolean parseDescLineIO(String key[],String value,boolean isInput, Tool t){
		Logger logger=Logger.getInstance();
		if(key[2].equals("type")) { //$NON-NLS-1$
			initPortType(value);
		}
		else if(key[2].equals("default")) { //$NON-NLS-1$
			setDefaultValue(value);
		}
		else if(key[2].equals("label")) { //$NON-NLS-1$
			setName(value);
		}
		else if(key[2].equals("comment")) { //$NON-NLS-1$
			setComment(value);
		}
		else {
			logger.log(
					Logger.WARNING,
					"Tool.parseDescLine: input("+key[2]+") subkey unhandled"); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
		
		if(isInput) {
			t.setInputPort(this.getID());
			
//			t.addInputPort(this.getID());
			System.out.println("id port : "+this.getID()+ " j "+this.hashCode());
		}
		else {
			t.setOutputPort(this.getID());
//			t.addOutputPort(this.getID());
			System.out.println("id port : "+this.getID());
		}
		
		return true;
	}
	
	public void initPortType(String type) {	
		String t = type.toLowerCase();
	
		System.out.println("initPortType : "+t);
		
		if(t.startsWith("fits")) {
			setData(new Constant(Utils.getIdUnique(),new Type("fits"),new Vector()));
		}
		else if(t.startsWith("numeric")) {
			if(t.startsWith("numeric:int")){
				setData(new Constant(Utils.getIdUnique(),new Type(t),new Integer(0)));
			}else if(t.startsWith("numeric:float")){
				setData(new Constant(Utils.getIdUnique(),new Type(t),new Float(0)));
			}else{
				setData(new Constant(Utils.getIdUnique(),new Type(t),new Double(0)));
			}
		}
		else if(t.startsWith("data")) {
			setData(new Constant(Utils.getIdUnique(),new Type(t),new Vector()));
		}
		else if(t.startsWith("image")) {
			setData(new Constant(Utils.getIdUnique(),new Type(t),new Vector()));
		}
		else if(t.startsWith("bool")) {
			setData(new Constant(Utils.getIdUnique(),Type.BOOLEAN,new Boolean(null)));
		}
	}
	
    /**
     * Copies and returns it.
     * @return a clone of this object
     */
    public Object clone() {
		ToolPort desc = new ToolPort(id, getName(), getPosition(),getComment());
		if(data!=null)
			desc.setData((Constant) data.clone());
		//desc.setAttable(new Hashtable(attable));
		System.out.println(this.getDefaultValue());
		if(defaultValue!=null){
			desc.setDefaultValue(defaultValue);
		}
		desc.setModules(new Hashtable(modules));
		
		return desc;
    }
	
	public String getComment(){
		return comment;
	}
	
	public void setComment(String comment){
		this.comment = comment;
	}
	

	
	public String getDefaultValue(){
		return defaultValue;
	}
	
	public void setDefaultValue(String dv){
		defaultValue = dv;
		if(dv==null) return;
		defaultValue=dv; // iokdbg: shld clone ?

		// addValue if !isReady might cause problem (unwanted initialization) ?
		if(!isReady()) {
			addValue(dv);
		}
	}
	
	public String getLabel(){
		return getName();
	}
	
	public String toString(){
		return getName();
	}
	
	public void setIndex(int index) {
		this.index = index;
	}

	public void addValue(String value) {
		Constant c = this.getData();
		if (c.getData() instanceof String || c.getData() instanceof Integer || c.getData() instanceof Double || c.getData() instanceof Float || c.getData() instanceof Boolean || c.getType().getName().startsWith("numeric")){
			c.setData(value);
		}else{
			Vector f = (Vector)c.getData();
			Vector nf = new Vector(f);
			nf.add(value);
			
			c.setData(nf);
		}
		obs.ping();
	}
	
	public void addValue(Boolean b){
		Constant c = this.getData();
		c.setData(b);
	}
	
	public void clearValues() {
		Constant c = this.getData();
		c.setData(new Vector());
		//f=new Vector();
		obs.ping();
	}
	public boolean isReady() {
		Constant c = this.getData();
		
		if (c.getData() instanceof String){
			if (!c.getData().equals(""))
				return true;
			return false;
		}else if (c.getData() instanceof Integer){
			if (!c.getData().equals(0))
				return true;
			return false;
		}else if (c.getData() instanceof Double){
			if (!c.getData().equals(0))
				return true;
			return false;
		}else if (c.getData() instanceof Float){
			if (!c.getData().equals(0))
				return true;
			return false;
		}else if (c.getType().getName().startsWith("numeric")){
			if (!c.getData().equals(0))
				return true;
			return false;
		}else if (c.getData() instanceof Boolean){
			if (!c.getData().equals(null))
				return true;
			return false;
		}else{
			Vector f = (Vector)c.getData();
			if(f!=null){
			if(f.size()!=0){
				return true;
			}else{
				return false;
			}
			}else{
				return false;
			}
		}
	}
	public boolean isMultivalued() {
		Enumeration e=getValues();
		boolean b=e.hasMoreElements();
		if(b) {
			e.nextElement();
			b=e.hasMoreElements();
		}
		return b;
	}
	
	/**
	 * @return first element of getValues() or null
	 */
	public Object getFirstValue() {
		Enumeration e=getValues();
		if(e.hasMoreElements()) {
			return e.nextElement();
		}
		return null;
	}
	
	public Enumeration getValues() {

		Constant c = this.getData();

		System.out.println("type getvalue : "+c.getData());
		if(c.getData() instanceof String) {
			String s = (String)c.getData();
			Vector v=new Vector();
			v.add((s==null)?"":s); //$NON-NLS-1$
			return v.elements();
		}else if(c.getData() instanceof Boolean){
			Boolean s = (Boolean)c.getData();
			Vector v=new Vector();
			if (s==null)
				s=false;
			v.add(s); //$NON-NLS-1$
			return v.elements();
		}else if(c.getData() instanceof Integer){
			Integer s = (Integer)c.getData();
			Vector v=new Vector();
			if (s==null)
				s=new Integer(0);
			v.add(s); //$NON-NLS-1$
			return v.elements();
		}else if(c.getData() instanceof Double){
			Double s = (Double)c.getData();
			Vector v=new Vector();
			if (s==null)
				s=new Double(0);
			v.add(s); //$NON-NLS-1$
			return v.elements();
		}else if(c.getData() instanceof Float){
			Float s = (Float)c.getData();
			Vector v=new Vector();
			if (s==null)
				s=new Float(0);
			v.add(s); //$NON-NLS-1$
			return v.elements();
		}else {
			Vector f = (Vector)c.getData();
			return f.elements();
		}
		
	}
	
	public void removeValueAt(int index) {
		try {
			Constant c = this.getData();
			Vector f = (Vector)c.getData();
			f.remove(index);
			c.setData(f);
		}
		catch(ArrayIndexOutOfBoundsException e) {
			Logger.getInstance().log(
					Logger.ERROR,
					"ToolIOFiles.removeValueAt: wrong index("+index+")"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		obs.ping();
	}
	
	public void removeValuesAt(int[] indices) {
		for(int i=0;i<indices.length;i++) {
			try {
				int x=indices[i];
				Constant c = this.getData();
				Vector f = (Vector)c.getData();
				f.remove(x);
				c.setData(f);
				// Indices shift in vector after remove(),
				// thus we adjust remaining indices.
				// Not elegant, but works.
				for(int j=i;j<indices.length;j++) {
					int y=indices[j];
					if(y>x) {
						indices[j]=y-1;
					}
				}
			}
			catch(ArrayIndexOutOfBoundsException e) {
				Logger.getInstance().log(
						Logger.ERROR,
						"ToolIOFiles.removeValuesAt: wrong index("+ //$NON-NLS-1$
						indices[i]+")"); //$NON-NLS-1$
			}
		}
		obs.ping();
	}
	
	public String valueAt(int index) {
		String s=""; //$NON-NLS-1$
		try {
			Constant c = this.getData();
			Vector f = (Vector)c.getData();
			s=f.elementAt(index).toString();
		}
		catch(Exception e) {
		}
		return s;
	}
	
	public void addObserver(Observer o) {
		obs.addObserver(o);
	}
	public void deleteObserver(Observer o) {
		obs.deleteObserver(o);
	}
	protected class ObservableToolPort extends Observable
	implements Serializable {
		private ToolPort t;
		ObservableToolPort(ToolPort t) {
			this.t=t;
		}
		void ping() {
			setChanged();
			notifyObservers();
		}
	}
}